@@ -1,30 +1,30 @@
-""" Multicast DNS Service Discovery for Python, v0.14-wmcbrine
-    Copyright 2003 Paul Scott-Murphy, 2014 William McBrine
+"""Multicast DNS Service Discovery for Python, v0.14-wmcbrine
+Copyright 2003 Paul Scott-Murphy, 2014 William McBrine
 
-    This module provides a framework for the use of DNS Service Discovery
-    using IP multicast.
+This module provides a framework for the use of DNS Service Discovery
+using IP multicast.
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-    Lesser General Public License for more details.
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
-    USA
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
+USA
 """
 
+from __future__ import annotations
+
 import asyncio
 import random
-import sys
-from ipaddress import IPv4Address, IPv6Address, _BaseAddress
-from typing import TYPE_CHECKING, Dict, List, Optional, Set, Union, cast
+from typing import TYPE_CHECKING, cast
 
 from .._cache import DNSCache
 from .._dns import (
@@ -50,6 +50,8 @@ from .._utils.asyncio import (
     wait_for_future_set_or_timeout,
 )
 from .._utils.ipaddress import (
+    ZeroconfIPv4Address,
+    ZeroconfIPv6Address,
     cached_ip_addresses,
     get_ip_address_object_from_record,
     ip_bytes_and_scope_to_address,
@@ -76,8 +78,6 @@ from ..const import (
     _TYPE_TXT,
 )
 
-IPADDRESS_SUPPORTS_SCOPE_ID = sys.version_info >= (3, 9, 0)
-
 _IPVersion_All_value = IPVersion.All.value
 _IPVersion_V4Only_value = IPVersion.V4Only.value
 # https://datatracker.ietf.org/doc/html/rfc6762#section-5.2
@@ -90,6 +90,10 @@ _IPVersion_V4Only_value = IPVersion.V4Only.value
 # the A/AAAA/SRV records for a host.
 _AVOID_SYNC_DELAY_RANDOM_INTERVAL = (20, 120)
 
+_TYPE_AAAA_RECORDS = {_TYPE_AAAA}
+_TYPE_A_RECORDS = {_TYPE_A}
+_TYPE_A_AAAA_RECORDS = {_TYPE_A, _TYPE_AAAA}
+
 bytes_ = bytes
 float_ = float
 int_ = int
@@ -104,7 +108,7 @@ if TYPE_CHECKING:
     from .._core import Zeroconf
 
 
-def instance_name_from_service_info(info: "ServiceInfo", strict: bool = True) -> str:
+def instance_name_from_service_info(info: ServiceInfo, strict: bool = True) -> str:
     """Calculate the instance name from the ServiceInfo."""
     # This is kind of funky because of the subtype based tests
     # need to make subtypes a first class citizen
@@ -137,45 +141,46 @@ class ServiceInfo(RecordUpdateListener):
     """
 
     __slots__ = (
-        "text",
-        "type",
-        "_name",
-        "key",
-        "_ipv4_addresses",
-        "_ipv6_addresses",
-        "port",
-        "weight",
-        "priority",
-        "server",
-        "server_key",
-        "_properties",
         "_decoded_properties",
-        "host_ttl",
-        "other_ttl",
-        "interface_index",
-        "_new_records_futures",
+        "_dns_address_cache",
         "_dns_pointer_cache",
         "_dns_service_cache",
         "_dns_text_cache",
-        "_dns_address_cache",
         "_get_address_and_nsec_records_cache",
+        "_ipv4_addresses",
+        "_ipv6_addresses",
+        "_name",
+        "_new_records_futures",
+        "_properties",
+        "_query_record_types",
+        "host_ttl",
+        "interface_index",
+        "key",
+        "other_ttl",
+        "port",
+        "priority",
+        "server",
+        "server_key",
+        "text",
+        "type",
+        "weight",
     )
 
     def __init__(
         self,
         type_: str,
         name: str,
-        port: Optional[int] = None,
+        port: int | None = None,
         weight: int = 0,
         priority: int = 0,
-        properties: Union[bytes, Dict] = b'',
-        server: Optional[str] = None,
+        properties: bytes | dict = b"",
+        server: str | None = None,
         host_ttl: int = _DNS_HOST_TTL,
         other_ttl: int = _DNS_OTHER_TTL,
         *,
-        addresses: Optional[List[bytes]] = None,
-        parsed_addresses: Optional[List[str]] = None,
-        interface_index: Optional[int] = None,
+        addresses: list[bytes] | None = None,
+        parsed_addresses: list[str] | None = None,
+        interface_index: int | None = None,
     ) -> None:
         # Accept both none, or one, but not both.
         if addresses is not None and parsed_addresses is not None:
@@ -183,12 +188,12 @@ class ServiceInfo(RecordUpdateListener):
         if not type_.endswith(service_type_name(name, strict=False)):
             raise BadTypeInNameException
         self.interface_index = interface_index
-        self.text = b''
+        self.text = b""
         self.type = type_
         self._name = name
         self.key = name.lower()
-        self._ipv4_addresses: List[IPv4Address] = []
-        self._ipv6_addresses: List[IPv6Address] = []
+        self._ipv4_addresses: list[ZeroconfIPv4Address] = []
+        self._ipv6_addresses: list[ZeroconfIPv6Address] = []
         if addresses is not None:
             self.addresses = addresses
         elif parsed_addresses is not None:
@@ -198,20 +203,21 @@ class ServiceInfo(RecordUpdateListener):
         self.priority = priority
         self.server = server if server else None
         self.server_key = server.lower() if server else None
-        self._properties: Optional[Dict[bytes, Optional[bytes]]] = None
-        self._decoded_properties: Optional[Dict[str, Optional[str]]] = None
+        self._properties: dict[bytes, bytes | None] | None = None
+        self._decoded_properties: dict[str, str | None] | None = None
         if isinstance(properties, bytes):
             self._set_text(properties)
         else:
             self._set_properties(properties)
         self.host_ttl = host_ttl
         self.other_ttl = other_ttl
-        self._new_records_futures: Optional[Set[asyncio.Future]] = None
-        self._dns_address_cache: Optional[List[DNSAddress]] = None
-        self._dns_pointer_cache: Optional[DNSPointer] = None
-        self._dns_service_cache: Optional[DNSService] = None
-        self._dns_text_cache: Optional[DNSText] = None
-        self._get_address_and_nsec_records_cache: Optional[Set[DNSRecord]] = None
+        self._new_records_futures: set[asyncio.Future] | None = None
+        self._dns_address_cache: list[DNSAddress] | None = None
+        self._dns_pointer_cache: DNSPointer | None = None
+        self._dns_service_cache: DNSService | None = None
+        self._dns_text_cache: DNSText | None = None
+        self._get_address_and_nsec_records_cache: set[DNSRecord] | None = None
+        self._query_record_types = {_TYPE_SRV, _TYPE_TXT, _TYPE_A, _TYPE_AAAA}
 
     @property
     def name(self) -> str:
@@ -220,7 +226,7 @@ class ServiceInfo(RecordUpdateListener):
 
     @name.setter
     def name(self, name: str) -> None:
-        """Replace the the name and reset the key."""
+        """Replace the name and reset the key."""
         self._name = name
         self.key = name.lower()
         self._dns_service_cache = None
@@ -228,7 +234,7 @@ class ServiceInfo(RecordUpdateListener):
         self._dns_text_cache = None
 
     @property
-    def addresses(self) -> List[bytes]:
+    def addresses(self) -> list[bytes]:
         """IPv4 addresses of this service.
 
         Only IPv4 addresses are returned for backward compatibility.
@@ -238,7 +244,7 @@ class ServiceInfo(RecordUpdateListener):
         return self.addresses_by_version(IPVersion.V4Only)
 
     @addresses.setter
-    def addresses(self, value: List[bytes]) -> None:
+    def addresses(self, value: list[bytes]) -> None:
         """Replace the addresses list.
 
         This replaces all currently stored addresses, both IPv4 and IPv6.
@@ -249,7 +255,7 @@ class ServiceInfo(RecordUpdateListener):
         self._get_address_and_nsec_records_cache = None
 
         for address in value:
-            if IPADDRESS_SUPPORTS_SCOPE_ID and len(address) == 16 and self.interface_index is not None:
+            if len(address) == 16 and self.interface_index is not None:
                 addr = ip_bytes_and_scope_to_address(address, self.interface_index)
             else:
                 addr = cached_ip_addresses(address)
@@ -260,15 +266,15 @@ class ServiceInfo(RecordUpdateListener):
                 )
             if addr.version == 4:
                 if TYPE_CHECKING:
-                    assert isinstance(addr, IPv4Address)
+                    assert isinstance(addr, ZeroconfIPv4Address)
                 self._ipv4_addresses.append(addr)
             else:
                 if TYPE_CHECKING:
-                    assert isinstance(addr, IPv6Address)
+                    assert isinstance(addr, ZeroconfIPv6Address)
                 self._ipv6_addresses.append(addr)
 
     @property
-    def properties(self) -> Dict[bytes, Optional[bytes]]:
+    def properties(self) -> dict[bytes, bytes | None]:
         """Return properties as bytes."""
         if self._properties is None:
             self._unpack_text_into_properties()
@@ -277,7 +283,7 @@ class ServiceInfo(RecordUpdateListener):
         return self._properties
 
     @property
-    def decoded_properties(self) -> Dict[str, Optional[str]]:
+    def decoded_properties(self) -> dict[str, str | None]:
         """Return properties as strings."""
         if self._decoded_properties is None:
             self._generate_decoded_properties()
@@ -293,7 +299,7 @@ class ServiceInfo(RecordUpdateListener):
         self._dns_text_cache = None
         self._get_address_and_nsec_records_cache = None
 
-    async def async_wait(self, timeout: float, loop: Optional[asyncio.AbstractEventLoop] = None) -> None:
+    async def async_wait(self, timeout: float, loop: asyncio.AbstractEventLoop | None = None) -> None:
         """Calling task waits for a given number of milliseconds or until notified."""
         if not self._new_records_futures:
             self._new_records_futures = set()
@@ -301,7 +307,7 @@ class ServiceInfo(RecordUpdateListener):
             loop or asyncio.get_running_loop(), self._new_records_futures, timeout
         )
 
-    def addresses_by_version(self, version: IPVersion) -> List[bytes]:
+    def addresses_by_version(self, version: IPVersion) -> list[bytes]:
         """List addresses matching IP version.
 
         Addresses are guaranteed to be returned in LIFO (last in, first out)
@@ -321,7 +327,7 @@ class ServiceInfo(RecordUpdateListener):
 
     def ip_addresses_by_version(
         self, version: IPVersion
-    ) -> Union[List[IPv4Address], List[IPv6Address], List[_BaseAddress]]:
+    ) -> list[ZeroconfIPv4Address] | list[ZeroconfIPv6Address]:
         """List ip_address objects matching IP version.
 
         Addresses are guaranteed to be returned in LIFO (last in, first out)
@@ -334,7 +340,7 @@ class ServiceInfo(RecordUpdateListener):
 
     def _ip_addresses_by_version_value(
         self, version_value: int_
-    ) -> Union[List[IPv4Address], List[IPv6Address]]:
+    ) -> list[ZeroconfIPv4Address] | list[ZeroconfIPv6Address]:
         """Backend for addresses_by_version that uses the raw value."""
         if version_value == _IPVersion_All_value:
             return [*self._ipv4_addresses, *self._ipv6_addresses]  # type: ignore[return-value]
@@ -342,7 +348,7 @@ class ServiceInfo(RecordUpdateListener):
             return self._ipv4_addresses
         return self._ipv6_addresses
 
-    def parsed_addresses(self, version: IPVersion = IPVersion.All) -> List[str]:
+    def parsed_addresses(self, version: IPVersion = IPVersion.All) -> list[str]:
         """List addresses in their parsed string form.
 
         Addresses are guaranteed to be returned in LIFO (last in, first out)
@@ -353,7 +359,7 @@ class ServiceInfo(RecordUpdateListener):
         """
         return [str_without_scope_id(addr) for addr in self._ip_addresses_by_version_value(version.value)]
 
-    def parsed_scoped_addresses(self, version: IPVersion = IPVersion.All) -> List[str]:
+    def parsed_scoped_addresses(self, version: IPVersion = IPVersion.All) -> list[str]:
         """Equivalent to parsed_addresses, with the exception that IPv6 Link-Local
         addresses are qualified with %<interface_index> when available
 
@@ -365,31 +371,31 @@ class ServiceInfo(RecordUpdateListener):
         """
         return [str(addr) for addr in self._ip_addresses_by_version_value(version.value)]
 
-    def _set_properties(self, properties: Dict[Union[str, bytes], Optional[Union[str, bytes]]]) -> None:
+    def _set_properties(self, properties: dict[str | bytes, str | bytes | None]) -> None:
         """Sets properties and text of this info from a dictionary"""
-        list_: List[bytes] = []
+        list_: list[bytes] = []
         properties_contain_str = False
-        result = b''
+        result = b""
         for key, value in properties.items():
             if isinstance(key, str):
-                key = key.encode('utf-8')
+                key = key.encode("utf-8")  # noqa: PLW2901
                 properties_contain_str = True
 
             record = key
             if value is not None:
                 if not isinstance(value, bytes):
-                    value = str(value).encode('utf-8')
+                    value = str(value).encode("utf-8")  # noqa: PLW2901
                     properties_contain_str = True
-                record += b'=' + value
+                record += b"=" + value
             list_.append(record)
         for item in list_:
-            result = b''.join((result, bytes((len(item),)), item))
+            result = b"".join((result, bytes((len(item),)), item))
         if not properties_contain_str:
             # If there are no str keys or values, we can use the properties
             # as-is, without decoding them, otherwise calling
             # self.properties will lazy decode them, which is expensive.
             if TYPE_CHECKING:
-                self._properties = cast("Dict[bytes, Optional[bytes]]", properties)
+                self._properties = cast(dict[bytes, bytes | None], properties)
             else:
                 self._properties = properties
         self.text = result
@@ -421,12 +427,12 @@ class ServiceInfo(RecordUpdateListener):
             return
 
         index = 0
-        properties: Dict[bytes, Optional[bytes]] = {}
+        properties: dict[bytes, bytes | None] = {}
         while index < end:
             length = text[index]
             index += 1
             key_value = text[index : index + length]
-            key_sep_value = key_value.partition(b'=')
+            key_sep_value = key_value.partition(b"=")
             key = key_sep_value[0]
             if key not in properties:
                 properties[key] = key_sep_value[2] or None
@@ -439,10 +445,10 @@ class ServiceInfo(RecordUpdateListener):
         return self._name[: len(self._name) - len(self.type) - 1]
 
     def _get_ip_addresses_from_cache_lifo(
-        self, zc: 'Zeroconf', now: float_, type: int_
-    ) -> List[Union[IPv4Address, IPv6Address]]:
+        self, zc: Zeroconf, now: float_, type: int_
+    ) -> list[ZeroconfIPv4Address | ZeroconfIPv6Address]:
         """Set IPv6 addresses from the cache."""
-        address_list: List[Union[IPv4Address, IPv6Address]] = []
+        address_list: list[ZeroconfIPv4Address | ZeroconfIPv6Address] = []
         for record in self._get_address_records_from_cache_by_type(zc, type):
             if record.is_expired(now):
                 continue
@@ -452,25 +458,27 @@ class ServiceInfo(RecordUpdateListener):
         address_list.reverse()  # Reverse to get LIFO order
         return address_list
 
-    def _set_ipv6_addresses_from_cache(self, zc: 'Zeroconf', now: float_) -> None:
+    def _set_ipv6_addresses_from_cache(self, zc: Zeroconf, now: float_) -> None:
         """Set IPv6 addresses from the cache."""
         if TYPE_CHECKING:
             self._ipv6_addresses = cast(
-                "List[IPv6Address]", self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_AAAA)
+                list[ZeroconfIPv6Address],
+                self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_AAAA),
             )
         else:
             self._ipv6_addresses = self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_AAAA)
 
-    def _set_ipv4_addresses_from_cache(self, zc: 'Zeroconf', now: float_) -> None:
+    def _set_ipv4_addresses_from_cache(self, zc: Zeroconf, now: float_) -> None:
         """Set IPv4 addresses from the cache."""
         if TYPE_CHECKING:
             self._ipv4_addresses = cast(
-                "List[IPv4Address]", self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_A)
+                list[ZeroconfIPv4Address],
+                self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_A),
             )
         else:
             self._ipv4_addresses = self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_A)
 
-    def async_update_records(self, zc: 'Zeroconf', now: float_, records: List[RecordUpdate]) -> None:
+    def async_update_records(self, zc: Zeroconf, now: float_, records: list[RecordUpdate]) -> None:
         """Updates service information from a DNS record.
 
         This method will be run in the event loop.
@@ -482,7 +490,7 @@ class ServiceInfo(RecordUpdateListener):
         if updated and new_records_futures:
             _resolve_all_futures_to_none(new_records_futures)
 
-    def _process_record_threadsafe(self, zc: 'Zeroconf', record: DNSRecord, now: float_) -> bool:
+    def _process_record_threadsafe(self, zc: Zeroconf, record: DNSRecord, now: float_) -> bool:
         """Thread safe record updating.
 
         Returns True if a new record was added.
@@ -507,24 +515,32 @@ class ServiceInfo(RecordUpdateListener):
 
             if ip_addr.version == 4:
                 if TYPE_CHECKING:
-                    assert isinstance(ip_addr, IPv4Address)
+                    assert isinstance(ip_addr, ZeroconfIPv4Address)
                 ipv4_addresses = self._ipv4_addresses
                 if ip_addr not in ipv4_addresses:
                     ipv4_addresses.insert(0, ip_addr)
                     return True
-                elif ip_addr != ipv4_addresses[0]:
+                # Use int() to compare the addresses as integers
+                # since by default IPv4Address.__eq__ compares the
+                # the addresses on version and int which more than
+                # we need here since we know the version is 4.
+                if ip_addr.zc_integer != ipv4_addresses[0].zc_integer:
                     ipv4_addresses.remove(ip_addr)
                     ipv4_addresses.insert(0, ip_addr)
 
                 return False
 
             if TYPE_CHECKING:
-                assert isinstance(ip_addr, IPv6Address)
+                assert isinstance(ip_addr, ZeroconfIPv6Address)
             ipv6_addresses = self._ipv6_addresses
             if ip_addr not in self._ipv6_addresses:
                 ipv6_addresses.insert(0, ip_addr)
                 return True
-            elif ip_addr != self._ipv6_addresses[0]:
+            # Use int() to compare the addresses as integers
+            # since by default IPv6Address.__eq__ compares the
+            # the addresses on version and int which more than
+            # we need here since we know the version is 6.
+            if ip_addr.zc_integer != self._ipv6_addresses[0].zc_integer:
                 ipv6_addresses.remove(ip_addr)
                 ipv6_addresses.insert(0, ip_addr)
 
@@ -561,17 +577,17 @@ class ServiceInfo(RecordUpdateListener):
 
     def dns_addresses(
         self,
-        override_ttl: Optional[int] = None,
+        override_ttl: int | None = None,
         version: IPVersion = IPVersion.All,
-    ) -> List[DNSAddress]:
+    ) -> list[DNSAddress]:
         """Return matching DNSAddress from ServiceInfo."""
         return self._dns_addresses(override_ttl, version)
 
     def _dns_addresses(
         self,
-        override_ttl: Optional[int],
+        override_ttl: int | None,
         version: IPVersion,
-    ) -> List[DNSAddress]:
+    ) -> list[DNSAddress]:
         """Return matching DNSAddress from ServiceInfo."""
         cacheable = version is IPVersion.All and override_ttl is None
         if self._dns_address_cache is not None and cacheable:
@@ -595,11 +611,11 @@ class ServiceInfo(RecordUpdateListener):
             self._dns_address_cache = records
         return records
 
-    def dns_pointer(self, override_ttl: Optional[int] = None) -> DNSPointer:
+    def dns_pointer(self, override_ttl: int | None = None) -> DNSPointer:
         """Return DNSPointer from ServiceInfo."""
         return self._dns_pointer(override_ttl)
 
-    def _dns_pointer(self, override_ttl: Optional[int]) -> DNSPointer:
+    def _dns_pointer(self, override_ttl: int | None) -> DNSPointer:
         """Return DNSPointer from ServiceInfo."""
         cacheable = override_ttl is None
         if self._dns_pointer_cache is not None and cacheable:
@@ -616,11 +632,11 @@ class ServiceInfo(RecordUpdateListener):
             self._dns_pointer_cache = record
         return record
 
-    def dns_service(self, override_ttl: Optional[int] = None) -> DNSService:
+    def dns_service(self, override_ttl: int | None = None) -> DNSService:
         """Return DNSService from ServiceInfo."""
         return self._dns_service(override_ttl)
 
-    def _dns_service(self, override_ttl: Optional[int]) -> DNSService:
+    def _dns_service(self, override_ttl: int | None) -> DNSService:
         """Return DNSService from ServiceInfo."""
         cacheable = override_ttl is None
         if self._dns_service_cache is not None and cacheable:
@@ -643,11 +659,11 @@ class ServiceInfo(RecordUpdateListener):
             self._dns_service_cache = record
         return record
 
-    def dns_text(self, override_ttl: Optional[int] = None) -> DNSText:
+    def dns_text(self, override_ttl: int | None = None) -> DNSText:
         """Return DNSText from ServiceInfo."""
         return self._dns_text(override_ttl)
 
-    def _dns_text(self, override_ttl: Optional[int]) -> DNSText:
+    def _dns_text(self, override_ttl: int | None) -> DNSText:
         """Return DNSText from ServiceInfo."""
         cacheable = override_ttl is None
         if self._dns_text_cache is not None and cacheable:
@@ -664,11 +680,11 @@ class ServiceInfo(RecordUpdateListener):
             self._dns_text_cache = record
         return record
 
-    def dns_nsec(self, missing_types: List[int], override_ttl: Optional[int] = None) -> DNSNsec:
+    def dns_nsec(self, missing_types: list[int], override_ttl: int | None = None) -> DNSNsec:
         """Return DNSNsec from ServiceInfo."""
         return self._dns_nsec(missing_types, override_ttl)
 
-    def _dns_nsec(self, missing_types: List[int], override_ttl: Optional[int]) -> DNSNsec:
+    def _dns_nsec(self, missing_types: list[int], override_ttl: int | None) -> DNSNsec:
         """Return DNSNsec from ServiceInfo."""
         return DNSNsec(
             self._name,
@@ -680,17 +696,17 @@ class ServiceInfo(RecordUpdateListener):
             0.0,
         )
 
-    def get_address_and_nsec_records(self, override_ttl: Optional[int] = None) -> Set[DNSRecord]:
+    def get_address_and_nsec_records(self, override_ttl: int | None = None) -> set[DNSRecord]:
         """Build a set of address records and NSEC records for non-present record types."""
         return self._get_address_and_nsec_records(override_ttl)
 
-    def _get_address_and_nsec_records(self, override_ttl: Optional[int]) -> Set[DNSRecord]:
+    def _get_address_and_nsec_records(self, override_ttl: int | None) -> set[DNSRecord]:
         """Build a set of address records and NSEC records for non-present record types."""
         cacheable = override_ttl is None
         if self._get_address_and_nsec_records_cache is not None and cacheable:
             return self._get_address_and_nsec_records_cache
-        missing_types: Set[int] = _ADDRESS_RECORD_TYPES.copy()
-        records: Set[DNSRecord] = set()
+        missing_types: set[int] = _ADDRESS_RECORD_TYPES.copy()
+        records: set[DNSRecord] = set()
         for dns_address in self._dns_addresses(override_ttl, IPVersion.All):
             missing_types.discard(dns_address.type)
             records.add(dns_address)
@@ -701,13 +717,16 @@ class ServiceInfo(RecordUpdateListener):
             self._get_address_and_nsec_records_cache = records
         return records
 
-    def _get_address_records_from_cache_by_type(self, zc: 'Zeroconf', _type: int_) -> List[DNSAddress]:
+    def _get_address_records_from_cache_by_type(self, zc: Zeroconf, _type: int_) -> list[DNSAddress]:
         """Get the addresses from the cache."""
         if self.server_key is None:
             return []
         cache = zc.cache
         if TYPE_CHECKING:
-            records = cast("List[DNSAddress]", cache.get_all_by_details(self.server_key, _type, _CLASS_IN))
+            records = cast(
+                list[DNSAddress],
+                cache.get_all_by_details(self.server_key, _type, _CLASS_IN),
+            )
         else:
             records = cache.get_all_by_details(self.server_key, _type, _CLASS_IN)
         return records
@@ -721,14 +740,14 @@ class ServiceInfo(RecordUpdateListener):
             self.server = self._name
             self.server_key = self.key
 
-    def load_from_cache(self, zc: 'Zeroconf', now: Optional[float_] = None) -> bool:
+    def load_from_cache(self, zc: Zeroconf, now: float_ | None = None) -> bool:
         """Populate the service info from the cache.
 
         This method is designed to be threadsafe.
         """
         return self._load_from_cache(zc, now or current_time_millis())
 
-    def _load_from_cache(self, zc: 'Zeroconf', now: float_) -> bool:
+    def _load_from_cache(self, zc: Zeroconf, now: float_) -> bool:
         """Populate the service info from the cache.
 
         This method is designed to be threadsafe.
@@ -758,10 +777,10 @@ class ServiceInfo(RecordUpdateListener):
 
     def request(
         self,
-        zc: 'Zeroconf',
+        zc: Zeroconf,
         timeout: float,
-        question_type: Optional[DNSQuestionType] = None,
-        addr: Optional[str] = None,
+        question_type: DNSQuestionType | None = None,
+        addr: str | None = None,
         port: int = _MDNS_PORT,
     ) -> bool:
         """Returns true if the service could be discovered on the
@@ -770,13 +789,22 @@ class ServiceInfo(RecordUpdateListener):
         While it is not expected during normal operation,
         this function may raise EventLoopBlocked if the underlying
         call to `async_request` cannot be completed.
+
+        :param zc: Zeroconf instance
+        :param timeout: time in milliseconds to wait for a response
+        :param question_type: question type to ask
+        :param addr: address to send the request to
+        :param port: port to send the request to
         """
-        assert zc.loop is not None and zc.loop.is_running()
+        assert zc.loop is not None, "Zeroconf instance must have a loop, was it not started?"
+        assert zc.loop.is_running(), "Zeroconf instance loop must be running, was it already stopped?"
         if zc.loop == get_running_loop():
             raise RuntimeError("Use AsyncServiceInfo.async_request from the event loop")
         return bool(
             run_coro_with_timeout(
-                self.async_request(zc, timeout, question_type, addr, port), zc.loop, timeout
+                self.async_request(zc, timeout, question_type, addr, port),
+                zc.loop,
+                timeout,
             )
         )
 
@@ -788,10 +816,10 @@ class ServiceInfo(RecordUpdateListener):
 
     async def async_request(
         self,
-        zc: 'Zeroconf',
+        zc: Zeroconf,
         timeout: float,
-        question_type: Optional[DNSQuestionType] = None,
-        addr: Optional[str] = None,
+        question_type: DNSQuestionType | None = None,
+        addr: str | None = None,
         port: int = _MDNS_PORT,
     ) -> bool:
         """Returns true if the service could be discovered on the
@@ -803,6 +831,12 @@ class ServiceInfo(RecordUpdateListener):
         mDNS multicast address and port. This is useful for directing
         requests to a specific host that may be able to respond across
         subnets.
+
+        :param zc: Zeroconf instance
+        :param timeout: time in milliseconds to wait for a response
+        :param question_type: question type to ask
+        :param addr: address to send the request to
+        :param port: port to send the request to
         """
         if not zc.started:
             await zc.async_wait_for_start()
@@ -882,7 +916,7 @@ class ServiceInfo(RecordUpdateListener):
             out.add_answer_at_time(answer, now)
 
     def _generate_request_query(
-        self, zc: 'Zeroconf', now: float_, question_type: DNSQuestionType
+        self, zc: Zeroconf, now: float_, question_type: DNSQuestionType
     ) -> DNSOutgoing:
         """Generate the request query."""
         out = DNSOutgoing(_FLAGS_QR_QUERY)
@@ -891,36 +925,86 @@ class ServiceInfo(RecordUpdateListener):
         cache = zc.cache
         history = zc.question_history
         qu_question = question_type is QU_QUESTION
-        self._add_question_with_known_answers(
-            out, qu_question, history, cache, now, name, _TYPE_SRV, _CLASS_IN, True
-        )
-        self._add_question_with_known_answers(
-            out, qu_question, history, cache, now, name, _TYPE_TXT, _CLASS_IN, True
-        )
-        self._add_question_with_known_answers(
-            out, qu_question, history, cache, now, server, _TYPE_A, _CLASS_IN, False
-        )
-        self._add_question_with_known_answers(
-            out, qu_question, history, cache, now, server, _TYPE_AAAA, _CLASS_IN, False
-        )
+        if _TYPE_SRV in self._query_record_types:
+            self._add_question_with_known_answers(
+                out, qu_question, history, cache, now, name, _TYPE_SRV, _CLASS_IN, True
+            )
+        if _TYPE_TXT in self._query_record_types:
+            self._add_question_with_known_answers(
+                out, qu_question, history, cache, now, name, _TYPE_TXT, _CLASS_IN, True
+            )
+        if _TYPE_A in self._query_record_types:
+            self._add_question_with_known_answers(
+                out, qu_question, history, cache, now, server, _TYPE_A, _CLASS_IN, False
+            )
+        if _TYPE_AAAA in self._query_record_types:
+            self._add_question_with_known_answers(
+                out, qu_question, history, cache, now, server, _TYPE_AAAA, _CLASS_IN, False
+            )
         return out
 
     def __repr__(self) -> str:
         """String representation"""
-        return '{}({})'.format(
+        return "{}({})".format(
             type(self).__name__,
-            ', '.join(
-                f'{name}={getattr(self, name)!r}'
+            ", ".join(
+                f"{name}={getattr(self, name)!r}"
                 for name in (
-                    'type',
-                    'name',
-                    'addresses',
-                    'port',
-                    'weight',
-                    'priority',
-                    'server',
-                    'properties',
-                    'interface_index',
+                    "type",
+                    "name",
+                    "addresses",
+                    "port",
+                    "weight",
+                    "priority",
+                    "server",
+                    "properties",
+                    "interface_index",
                 )
             ),
         )
+
+
+class AsyncServiceInfo(ServiceInfo):
+    """An async version of ServiceInfo."""
+
+
+class AddressResolver(ServiceInfo):
+    """Resolve a host name to an IP address."""
+
+    def __init__(self, server: str) -> None:
+        """Initialize the AddressResolver."""
+        super().__init__(server, server, server=server)
+        self._query_record_types = _TYPE_A_AAAA_RECORDS
+
+    @property
+    def _is_complete(self) -> bool:
+        """The ServiceInfo has all expected properties."""
+        return bool(self._ipv4_addresses) or bool(self._ipv6_addresses)
+
+
+class AddressResolverIPv6(ServiceInfo):
+    """Resolve a host name to an IPv6 address."""
+
+    def __init__(self, server: str) -> None:
+        """Initialize the AddressResolver."""
+        super().__init__(server, server, server=server)
+        self._query_record_types = _TYPE_AAAA_RECORDS
+
+    @property
+    def _is_complete(self) -> bool:
+        """The ServiceInfo has all expected properties."""
+        return bool(self._ipv6_addresses)
+
+
+class AddressResolverIPv4(ServiceInfo):
+    """Resolve a host name to an IPv4 address."""
+
+    def __init__(self, server: str) -> None:
+        """Initialize the AddressResolver."""
+        super().__init__(server, server, server=server)
+        self._query_record_types = _TYPE_A_RECORDS
+
+    @property
+    def _is_complete(self) -> bool:
+        """The ServiceInfo has all expected properties."""
+        return bool(self._ipv4_addresses)
