@@ -1,5 +1,7 @@
 """Unit tests for zeroconf._handlers"""
 
+from __future__ import annotations
+
 import asyncio
 import logging
 import os
@@ -7,7 +9,7 @@ import socket
 import time
 import unittest
 import unittest.mock
-from typing import List, cast
+from typing import cast
 from unittest.mock import patch
 
 import pytest
@@ -65,10 +67,9 @@ class TestRegistrar(unittest.TestCase):
         def get_ttl(record_type):
             if expected_ttl is not None:
                 return expected_ttl
-            elif record_type in [const._TYPE_A, const._TYPE_SRV, const._TYPE_NSEC]:
+            if record_type in [const._TYPE_A, const._TYPE_SRV, const._TYPE_NSEC]:
                 return const._DNS_HOST_TTL
-            else:
-                return const._DNS_OTHER_TTL
+            return const._DNS_OTHER_TTL
 
         def _process_outgoing_packet(out):
             """Sends an outgoing packet."""
@@ -1371,7 +1372,7 @@ async def test_record_update_manager_add_listener_callsback_existing_records():
     class MyListener(r.RecordUpdateListener):
         """A RecordUpdateListener that does not implement update_records."""
 
-        def async_update_records(self, zc: "Zeroconf", now: float, records: List[r.RecordUpdate]) -> None:
+        def async_update_records(self, zc: Zeroconf, now: float, records: list[r.RecordUpdate]) -> None:
             """Update multiple records in one shot."""
             updated.extend(records)
 
@@ -1973,7 +1974,7 @@ async def test_add_listener_warns_when_not_using_record_update_listener(caplog):
     class MyListener:
         """A RecordUpdateListener that does not implement update_records."""
 
-        def async_update_records(self, zc: "Zeroconf", now: float, records: List[r.RecordUpdate]) -> None:
+        def async_update_records(self, zc: Zeroconf, now: float, records: list[r.RecordUpdate]) -> None:
             """Update multiple records in one shot."""
             updated.extend(records)
 
@@ -2005,7 +2006,7 @@ async def test_async_updates_iteration_safe():
     class OtherListener(r.RecordUpdateListener):
         """A RecordUpdateListener that does not implement update_records."""
 
-        def async_update_records(self, zc: "Zeroconf", now: float, records: List[r.RecordUpdate]) -> None:
+        def async_update_records(self, zc: Zeroconf, now: float, records: list[r.RecordUpdate]) -> None:
             """Update multiple records in one shot."""
             updated.extend(records)
 
@@ -2014,7 +2015,7 @@ async def test_async_updates_iteration_safe():
     class ListenerThatAddsListener(r.RecordUpdateListener):
         """A RecordUpdateListener that does not implement update_records."""
 
-        def async_update_records(self, zc: "Zeroconf", now: float, records: List[r.RecordUpdate]) -> None:
+        def async_update_records(self, zc: Zeroconf, now: float, records: list[r.RecordUpdate]) -> None:
             """Update multiple records in one shot."""
             updated.extend(records)
             zc.async_add_listener(other, None)
