@@ -1,11 +1,12 @@
 """Unit tests for zeroconf._protocol"""
 
+from __future__ import annotations
+
 import copy
 import logging
 import os
 import socket
 import struct
-import unittest
 import unittest.mock
 from typing import cast
 
@@ -707,7 +708,7 @@ def test_tc_bit_in_query_packet():
     for i in range(30):
         out.add_answer_at_time(
             DNSText(
-                ("HASS Bridge W9DN %s._hap._tcp.local." % i),
+                f"HASS Bridge W9DN {i}._hap._tcp.local.",
                 const._TYPE_TXT,
                 const._CLASS_IN | const._CLASS_UNIQUE,
                 const._DNS_OTHER_TTL,
@@ -739,7 +740,7 @@ def test_tc_bit_not_set_in_answer_packet():
     for i in range(30):
         out.add_answer_at_time(
             DNSText(
-                ("HASS Bridge W9DN %s._hap._tcp.local." % i),
+                f"HASS Bridge W9DN {i}._hap._tcp.local.",
                 const._TYPE_TXT,
                 const._CLASS_IN | const._CLASS_UNIQUE,
                 const._DNS_OTHER_TTL,
@@ -813,7 +814,7 @@ def test_records_same_packet_share_fate():
     for i in range(30):
         out.add_answer_at_time(
             DNSText(
-                ("HASS Bridge W9DN %s._hap._tcp.local." % i),
+                f"HASS Bridge W9DN {i}._hap._tcp.local.",
                 const._TYPE_TXT,
                 const._CLASS_IN | const._CLASS_UNIQUE,
                 const._DNS_OTHER_TTL,
